<?php

namespace ShineSoftware\Constriv\Gateway\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Store\Model\ScopeInterface;
use ShineSoftware\Constriv\Model\Source\CurrenciesFactory;

class AuthorizationRequest implements BuilderInterface {

    private $config;
    public $_storeManager;
    public $_resolver;
    public $_logger;
    protected $_sourceCurrenciesFactory;

    public function __construct(
    ConfigInterface $config, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Framework\Locale\Resolver $resolver, \ShineSoftware\Constriv\Model\Logger\Shinelogger $logger, CurrenciesFactory $sourceCurrenciesFactory
    ) {
        $this->config = $config;
        $this->_storeManager = $storeManager;
        $this->_resolver = $resolver;
        $this->_logger = $logger;
        $this->_sourceCurrenciesFactory = $sourceCurrenciesFactory;
    }

    public function build(array $buildSubject) {
        if (!isset($buildSubject['payment']) || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();
        if ($this->config->getValue('debug', $order->getStoreId())) {
            $mode = "Test Mode";
        } else {
            $mode = "Live Mode";
        }
        $orderId = $order->getOrderIncrementId();
        $currencies = $this->_sourceCurrenciesFactory->create();
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Consorzio Triveneto Starts - Order: $orderId - Merchant Id (" . $this->config->getValue('id', $order->getStoreId()) . ") - $mode");
        $data = [
            'id' => $this->config->getValue('id', $order->getStoreId()),
            'password' => $this->config->getValue('password', $order->getStoreId()),
            'page' => 'MOBILE',
            'amt' => number_format($order->getGrandTotalAmount(), 2, '.', ''),
            'currencycode' => $currencies->findCurrencyIsoCode($order->getCurrencyCode()),
            'trackid' => $order->getOrderIncrementId(),
            'langid' => 'ITA',
            'responseURL' => $this->_storeManager->getStore()->getBaseUrl() . 'constriv/hosted/response',
//            'responseURL' => 'http://fd7d45e0.ngrok.io/constriv/hosted/response',
            'errorURL' => $this->_storeManager->getStore()->getBaseUrl() . 'constriv/hosted/error',
            'member' => $order->getBillingAddress()->getFirstname() . " " . $order->getBillingAddress()->getLastname(),
            'udf3' => "EMAILADDR:" . $order->getBillingAddress()->getEmail(),
            'url' => $this->getEndpointUrl('payment_init', $order->getStoreId()),
        ];
        if ($this->config->getValue('credit_card_sha', $order->getStoreId())) {
            $data['udf1'] = "SHA1";
        }
        if ($this->config->getValue('hpp_timeout', $order->getStoreId())) {
            $timeout = $this->config->getValue('hpp_timeout', $order->getStoreId());
            if (is_numeric($timeout)) {
                $data['udf5'] = "HPP_TIMEOUT=$timeout";
            } else {
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Wrong Timeout setting! Please check the configuration panel and fix the error!");
            }
        }
        if ("authorize_capture" == $this->config->getValue('request_type', $order->getStoreId())) {
            $data['action'] = 1;
        } else {
            $data['action'] = 4;
        }
        $this->_logger->debug(__METHOD__ . " - " . json_encode($data));
        return $data;
    }
    
    
    private function getEndpointUrl($type = 'payment_init', $storeId = 1) {
        if ($this->config->getValue("debug", $storeId)) {
            return $this->config->getValue("uri/test/$type", $storeId);
        } else {
            return $this->config->getValue("uri/live/$type", $storeId);
        }
    }

}

;
